<?php
/**
* Plugin Name: Google Shopping XML Feed
* Description: A plugin to generate a Google Shopping XML Feed for Woocommerce products.
* Version: 1.0
* Author: Shanan Howe
**/

// adds plugin option to the admin menu
add_action('admin_menu', 'gmf_feed');
 
function gmf_feed() {
    add_menu_page('Google Shopping Feed', 'Google Shopping', 'manage_options', 'googleshopping-plugin', 'gmf_settings','dashicons-cart');
}

add_action( 'woocommerce_product_options_general_product_data', 'gmf_infeed' );

// Add the GMF checkbox to every product
function gmf_infeed() {

	global $post;

	if(get_post_meta($post->ID,'gmf_infeed',true) == "yes"):
		$checked = 'checked';
	endif;

	echo '<p class="form-field gmf_infeed">';
		echo '<label for="gmf">Google Shopping Feed</label> <input type="checkbox" name="gmf_infeed" value="yes" '.$checked.'>';
	echo '</p>';

} 

// On theme activation cycle through every woocommerce product and add it to the shopping feed.
function gmf_plugin_activation() {

	global $wpdb;

	$sql = "
	    SELECT $wpdb->posts.* 
	    FROM $wpdb->posts 
	    WHERE $wpdb->posts.post_type = 'product'
	";

	$products = $wpdb->get_results($sql);
	
	foreach($products as $product):

		// Only add the product to the Shopping feed if it has no existing post meta
		if(!get_post_meta($product->ID,'gmf_infeed',true)):
			update_post_meta($product->ID,'gmf_infeed','yes');
		endif;

	endforeach;

}
register_activation_hook( __FILE__, 'gmf_plugin_activation' );

// Save the checkbox if its ticked
function save_gmf_infeed( $post_id ) {

	if(empty($_POST['gmf_infeed'])):
		update_post_meta($post_id,'gmf_infeed','no');
	else: 
		update_post_meta($post_id,'gmf_infeed','yes');
	endif;

}
add_action( 'save_post', 'save_gmf_infeed' );

// generate the Google Shopping XML Feed and only show it on the /shopping-feed url
function gmf_generate() {

	if(str_replace('/','',$_SERVER['REQUEST_URI']) == "shopping-feed"):

		// plugin custom fields
		$title = get_option('gmf_title');
		$description = get_option('gmf_description');
		$link = get_option('gmf_link');
		$productcategory = get_option('gmf_productcategory');

		global $wpdb;

		$sql = "
		    SELECT $wpdb->posts.* 
		    FROM $wpdb->posts, $wpdb->postmeta
		    WHERE $wpdb->posts.ID = $wpdb->postmeta.post_id
		    AND $wpdb->postmeta.meta_key = 'gmf_infeed' 
		    AND $wpdb->postmeta.meta_value = 'yes' 
		    AND $wpdb->posts.post_status = 'publish' 
		    AND $wpdb->posts.post_type = 'product'
		";

		$results = $wpdb->get_results($sql);

		header('content-type: text/xml');
		
		// build the xml header information to get that out of the way
		$output = '<?xml version="1.0" encoding="UTF-8" ?>';
		$output .= '<rss version="2.0" xmlns:g="http://base.google.com/ns/1.0" xmlns:c="http://base.google.com/cns/1.0">';
		$output .= '<channel>';
		$output .= '<title>' . $title . '</title>';
		$output .= '<link>'.$link.'</link>';
		$output .= '<description>'. $description.'</description>';

		// output each woocommerce product
		foreach($results as $result):
		
			// output the xml for each individual product
			$output .= '<item>';
				$output .= '<title>'.wptexturize($result->post_title).'</title>';
				$output .= '<link>'.$result->guid.'</link>';
				$output .= '<description>'.wptexturize(strip_tags($result->post_excerpt)).'</description>';
				//$output .= '<description>'.wptexturize(strip_tags($result->post_content)).'</description>'; You may wish to use some other functions depending on the content
				$output .= '<g:id>'.$result->ID.'</g:id>';
				$output .= '<g:brand>'.$title.'</g:brand>';
				$output .= '<g:mpn>GMF_'.$result->ID.'</g:mpn>';
				$output .= '<g:product_type>'.$productcategory.'</g:product_type>';
				$output .= '<g:google_product_category>'.$productcategory.'</g:google_product_category>';

				if(get_post_meta($result->ID, "_price", true)) {
				$output .= '<g:price>'.get_post_meta($result->ID, "_price", true).'</g:price>';
				}

				if(get_post_meta($result->ID, "_sale_price", true)) {
				    $output .= '<g:sale_price>'.get_post_meta($result->ID, "_sale_price", true).'</g:sale_price>';
				}
				if(get_post_meta($result->ID, "_sale_price_dates_from", true)):
					$output .= '<g:sale_price_effective_date>'.date('Y-m-d',get_post_meta($result->ID, "_sale_price_dates_from", true)).'/'.date('Y-m-d',get_post_meta($result->ID, "_sale_price_dates_to", true)).'</g:sale_price_effective_date>';
				endif;

				// the image of the product
				$image = wp_get_attachment_image_src( get_post_thumbnail_id( $result->ID ), 'full');
				$output .= '<g:image_link>'.$image[0].'</g:image_link>';

				// Stock Availability
				if(get_post_meta($result->ID, '_stock_status', true) == "instock"):
					$stock_status = 'in stock';
				else: 
					$stock_status = 'out of stock';
				endif;

				$output .= '<g:availability>'.$stock_status.'</g:availability>';
				$output .= '<g:condition>new</g:condition>';
			$output .= '</item>';

		endforeach;
		
		// end the feed by closing everything out
		$output .= '</channel>';
		$output .= '</rss>';

		echo $output;
		exit;
	
	endif;

}

// Plugin settings page
function gmf_settings(){

	// plugin custom fields
	$title = get_option('gmf_title');
	$description = get_option('gmf_description');
	$link = get_option('gmf_link');
	$productcategory = get_option('gmf_productcategory');

	if($_POST['Update']):

		echo '<div class="success"><h2>Success</h2>';
			echo '<p>The Google Shopping Feed options have been successfully updated @ '.date("h:i:sa").'</p>';
		echo '</div>';

		$title = wptexturize($_POST['title']);
		update_option('gmf_title', mysql_real_escape_string($title));

		$description = wptexturize($_POST['description']);
		update_option('gmf_description', mysql_real_escape_string($description));

		$link = wptexturize($_POST['link']);
		update_option('gmf_link', mysql_real_escape_string($link));

		if(empty($link)):
			$link = site_url();
		endif;

		$productcategory = mysql_real_escape_string(wptexturize($_POST['productcategory']));
		update_option('gmf_productcategory', $productcategory);

	endif;
	?>
	<div class="gmfwrap">
	    <form name="gmfform" method="post" action="<?php echo str_replace( '%7E', '~', $_SERVER['REQUEST_URI']); ?>">

	        <h2>Google Shopping XML Feed</h2>

	        <p>By default all of your Woocommerce products will appear in the Google Shopping XML feed. If you wish to remove them from the feed you can untick the 'Google Shopping Feed' checkbox on the edit product screen.

	        <p><strong>Feed XML URL:</strong><br/>
				<a href="<?php echo site_url().'/shopping-feed'; ?>"><?php echo site_url().'/shopping-feed'; ?></a>
			</p>

			<p><strong>Feed Title:</strong><br/><input type="text" name="title" value="<?php echo $title; ?>" size="70"></p>
			<p><strong>Feed Description:</strong><br/><textarea name="description"><?php echo $description; ?></textarea></p>
			<p><strong>Site URL:</strong><br/><input type="text" name="link" value="<?php echo $link; ?>" size="70"></p>
	        <p><strong>Google Product Category:</strong><br/><input type="text" name="productcategory" value="<?php echo $productcategory; ?>" size="70"> <a href="http://www.google.com/basepages/producttype/taxonomy.en-US.txt" target="_blank">See Google's Category List</a></p>

	        <p><input type="submit" name="Update" value="Update" /></p>

	    </form>
	</div>

	<!-- Admin Styles -->
	<style>
	input[type='text'],textarea { 
		width: 600px; 
	}
	textarea { 
		height: 100px;
	}
	.success { 
		background: #ddd; 
		padding: 5px; 
		padding-left: 20px; 
		padding-right: 20px; 
		margin-top: 10px;
		padding-top: 5px; 
		width: auto; 
	} 
	</style>
<?php
}
echo gmf_generate();
?>